@@ -54,6 +54,8 @@ from typing_extensions import Self
 from data_diff.abcs.compiler import AbstractCompiler, Compilable
 from data_diff.abcs.database_types import (
     JSON,
+    ArithAlphanumeric,
+    ArithUnicodeString,
     Array,
     Boolean,
     ColType,
@@ -114,7 +116,11 @@ from data_diff.queries.extras import (
 )
 from data_diff.schema import RawColumnInfo
 from data_diff.utils import (
+    ArithDate,
+    ArithDateTime,
     ArithString,
+    ArithTimestamp,
+    ArithTimestampTZ,
     ArithUUID,
     SybaseDriverTypes,
     is_uuid,
@@ -264,6 +270,7 @@ class BaseDialect(abc.ABC):
             "freetds_query_chosen": False,
         }
     )
+    table_schema: Dict[str, RawColumnInfo] = attrs.Factory(dict)
 
     def enable_preventing_type_overflow(self) -> None:
         logger.info("Preventing type overflow when concatenation is enabled")
@@ -752,6 +759,18 @@ class BaseDialect(abc.ABC):
             return f"'{v.decode()}'"
         elif isinstance(v, Code):
             return v.code
+        elif isinstance(v, ArithAlphanumeric):
+            return f"'{v._str}'"
+        elif isinstance(v, ArithUnicodeString):
+            return f"'{v._str}'"
+        elif isinstance(v, ArithDate):
+            return f"'{str(v)}'"
+        elif isinstance(v, ArithTimestamp):
+            return f"'{str(v)}'"
+        elif isinstance(v, ArithTimestampTZ):
+            return f"'{str(v)}'"
+        elif isinstance(v, ArithDateTime):
+            return f"'{str(v)}'"
         return repr(v)
 
     def constant_values(self, rows) -> str:
@@ -956,6 +975,10 @@ class BaseDialect(abc.ABC):
         schema_prefix = f"{schema}." if schema else ""
         return f"DROP VIEW {schema_prefix}{view_name}"
 
+    def get_column_raw_info(self, value: str) -> Optional[RawColumnInfo]:
+        raw_column_info = self.table_schema.get(value, None)
+        return raw_column_info
+
 
 T = TypeVar("T", bound=BaseDialect)
 Row = Sequence[Any]
@@ -1122,6 +1145,8 @@ class Database(abc.ABC):
             for r in rows
         }
         assert len(d) == len(rows)
+        if not self.dialect.table_schema:
+            self.dialect.table_schema = d
         return d
 
     def select_table_unique_columns(self, path: DbPath) -> str:
