@@ -12,6 +12,7 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import re
 import time
 from typing import Any, ClassVar, Dict, List, Optional, Tuple, Type
 
@@ -57,12 +58,23 @@ def import_sybase():
     return pyodbc
 
 
+def generate_primes(limit: int) -> List[int]:
+    sieve = [True] * (limit + 1)
+    sieve[0:2] = [False, False]
+    for i in range(2, int(limit**0.5) + 1):
+        if sieve[i]:
+            sieve[i * i : limit + 1 : i] = [False] * len(range(i * i, limit + 1, i))
+    return [i for i, is_prime in enumerate(sieve) if is_prime]
+
+
 @attrs.define(frozen=False)
 class Dialect(BaseDialect):
     name = "Sybase"
     ROUNDS_ON_PREC_LOSS = True
     SUPPORTS_PRIMARY_KEY: ClassVar[bool] = True
     SUPPORTS_INDEXES = True
+    primes: List[int] = attrs.Factory(lambda: generate_primes(1000))
+    column_prime_map: Dict[str, int] = attrs.Factory(dict)
     TYPE_CLASSES = {
         # Timestamps
         "datetimeoffset": TimestampTZ,
@@ -129,13 +141,17 @@ class Dialect(BaseDialect):
         WHERE name = CURRENT_USER"""
 
     def to_string(self, s: str, coltype: str = None) -> str:
+        s_temp = re.sub(r'["\[\]`]', "", s)
+        raw_col_info = self.get_column_raw_info(s_temp)
+        ch_len = (raw_col_info and raw_col_info.character_maximum_length) or None
+        if not ch_len:
+            ch_len = 2500
+        ch_len = max(ch_len, 2500)
         if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
-            # Sybase IQ or FreeTDS detected as IQ: Use VARCHAR(2500)
-            return f"CAST({s} AS VARCHAR(2500))"
-        # Sybase ASE or FreeTDS detected as ASE: Handle nvarchar
-        if coltype in ("nvarchar", "nchar", "ntext"):
-            return f"CAST({s} AS NVARCHAR(5000))"  # ASE max for nvarchar
-        return f"CAST({s} AS VARCHAR(2500))"
+            return f"CAST({s} AS VARCHAR({ch_len}))"
+        if raw_col_info and raw_col_info.data_type in ["nvarchar", "nchar", "ntext"]:
+            return f"CAST({s} AS NVARCHAR({ch_len}))"
+        return f"CAST({s} AS VARCHAR({ch_len}))"
 
     def type_repr(self, t) -> str:
         try:
@@ -158,23 +174,21 @@ class Dialect(BaseDialect):
         limit: Optional[int] = None,
         has_order_by: Optional[bool] = None,
     ) -> str:
-        import re
+        # import re
 
-        def safe_trim(match):
-            column_name = match.group(1)
-            if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
-                return f"TRIM(CAST({column_name} AS VARCHAR(2500)))"
-            return f"TRIM(CAST({column_name} AS NVARCHAR(5000)))"
-
-        select_query = re.sub(r"TRIM\(\[([\w]+)\]\)", safe_trim, select_query)
-
-        select_query = re.sub(r"TRIM\(([\w]+)\)", safe_trim, select_query)
+        # def safe_trim(match):
+        #     column_name = match.group(1)
+        #     if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
+        #         return f"TRIM(CAST({column_name} AS VARCHAR(2500)))"
+        #     return f"TRIM(CAST({column_name} AS NVARCHAR(5000)))"
+        # select_query = re.sub(r"TRIM\(\[([\w]+)\]\)", safe_trim, select_query)
+        # select_query = re.sub(r"TRIM\(([\w]+)\)", safe_trim, select_query)
 
         if limit is not None:
             select_query = select_query.replace("SELECT", f"SELECT TOP {limit}", 1)
 
-        if not has_order_by:
-            select_query += " ORDER BY 1"
+        # if not has_order_by:
+        #     select_query += " ORDER BY RAND()"
         return select_query
 
     def constant_values(self, rows) -> str:
@@ -183,6 +197,8 @@ class Dialect(BaseDialect):
 
     def normalize_timestamp(self, value: str, coltype: TemporalType) -> str:
         if isinstance(coltype, Datetime):
+            if coltype.precision == 4:
+                return f"CAST({value} AS VARCHAR(100))"
             if coltype.precision > 0:
                 return (
                     f"CASE WHEN {value} IS NULL THEN NULL "
@@ -208,18 +224,100 @@ class Dialect(BaseDialect):
                 f"END"
             )
         if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
-            return f"CAST({value} AS VARCHAR(2500))"
-        return f"CAST({value} AS NVARCHAR(5000))"
+            return f"CAST({value} AS VARCHAR(100))"
+        return f"CAST({value} AS NVARCHAR(100))"
 
     def normalize_number(self, value: str, coltype: FractionalType) -> str:
         return self.to_string(f"CAST({value} AS DECIMAL(38, {coltype.precision}))")
 
+    # def md5_as_int(self, s: str) -> str:
+    #     """Returns an MD5 hash of the input string as an integer for Sybase IQ."""
+    #     return f"CAST(HEXTOINT(LEFT(CAST(HASH({s}, 'MD5') AS VARCHAR(32)), 8)) AS BIGINT) - 140737488355327"
+
+    # def md5_as_int(self, s: str) -> str:
+    #     """Returns a hash-like integer based on ASCII values of the input string for Sybase."""
+    #     # Create a simple hash using ASCII values and string length
+    #     # This generates a pseudo-hash by combining ASCII values with position weights
+    #     return (
+    #         f"CAST(("
+    #         f"  (LEN({s}) * 31) + "  # Length component
+    #         f"  (ASCII(LEFT({s}, 1)) * 97) + "  # First character
+    #         f"  (CASE WHEN LEN({s}) > 1 THEN ASCII(SUBSTRING({s}, 2, 1)) * 53 ELSE 0 END) + "  # Second character
+    #         f"  (CASE WHEN LEN({s}) > 2 THEN ASCII(SUBSTRING({s}, 3, 1)) * 29 ELSE 0 END) + "  # Third character
+    #         f"  (CASE WHEN LEN({s}) > 3 THEN ASCII(RIGHT({s}, 1)) * 17 ELSE 0 END)"  # Last character
+    #         f") % 2147483647 AS BIGINT) - 1073741823"  # Modulo to keep in range and shift
+    #     )
+
+    # def md5_as_hex(self, s: str) -> str:
+    #     return f"HashBytes('MD5', {s})"
+
+    # def md5_as_hex(self, s: str) -> str:
+    #     """Returns a hex representation based on ASCII values instead of MD5."""
+    #     # Create a hex-like string using ASCII values
+    #     return (
+    #         f"RIGHT('0000000' + CONVERT(VARCHAR(8), "
+    #         f"  (ASCII(LEFT({s}, 1)) * 256 + "
+    #         f"   CASE WHEN LEN({s}) > 1 THEN ASCII(SUBSTRING({s}, 2, 1)) ELSE 0 END) % 65536"
+    #         f"), 16), 8)"
+    #     )
+
+    def get_unique_prime_for_column(self, column_name: str) -> int:
+        if column_name in self.column_prime_map:
+            return self.column_prime_map[column_name]
+        used_primes = set(self.column_prime_map.values())
+        for p in self.primes:
+            if p > 100 and p not in used_primes:
+                self.column_prime_map[column_name] = p
+                return p
+        raise ValueError("Ran out of unique primes")
+
     def md5_as_int(self, s: str) -> str:
-        """Returns an MD5 hash of the input string as an integer for Sybase IQ."""
-        return f"CAST(HEXTOINT(LEFT(CAST(HASH({s}, 'MD5') AS VARCHAR(32)), 8)) AS BIGINT) - 140737488355327"
+        if self.sybase_driver_type.is_ase or self.query_config_for_free_tds["ase_query_chosen"]:
+            return f"CAST(HEXTOINT(LEFT(CAST(HASH({s}, 'MD5') AS VARCHAR(32)), 8)) AS BIGINT) - 140737488355327"
+        base_prime = self.get_unique_prime_for_column(s)
+        separator = " +\n        "
+        parts = [f"LENGTH(COALESCE({s}, '')) * {base_prime}"]
+
+        for i in range(15):
+            parts.append(f"COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), {i + 1}, 1)), 0) * {self.primes[i]}")
+
+        for i, pos in enumerate([20, 25, 30, 35, 40]):
+            parts.append(
+                f"(CASE WHEN LENGTH(COALESCE({s}, '')) >= {pos} "
+                f"THEN COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), {pos}, 1)), 0) * {self.primes[15 + i]} ELSE 0 END)"
+            )
+
+        parts.append(
+            f"(CASE WHEN LENGTH(COALESCE({s}, '')) > 15 "
+            f"THEN COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), LENGTH(COALESCE({s}, '')), 1)), 0) * {self.primes[20]} ELSE 0 END)"
+        )
+
+        return f"CAST((\n        {separator.join(parts)}\n      ) % 2147483647 AS BIGINT)"
 
     def md5_as_hex(self, s: str) -> str:
-        return f"HashBytes('MD5', {s})"
+        if self.sybase_driver_type.is_ase or self.query_config_for_free_tds["ase_query_chosen"]:
+            return f"HashBytes('MD5', {s})"
+        base_prime = self.get_unique_prime_for_column(s)
+        separator = " +\n        "
+        parts = [f"LENGTH(COALESCE({s}, '')) * {base_prime}"]
+
+        for i in range(15):
+            parts.append(f"COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), {i + 1}, 1)), 0) * {self.primes[i]}")
+
+        for i, pos in enumerate([20, 25, 30, 35, 40]):
+            parts.append(
+                f"(CASE WHEN LENGTH(COALESCE({s}, '')) >= {pos} "
+                f"THEN COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), {pos}, 1)), 0) * {self.primes[15 + i]} ELSE 0 END)"
+            )
+
+        parts.append(
+            f"(CASE WHEN LENGTH(COALESCE({s}, '')) > 15 "
+            f"THEN COALESCE(ASCII(SUBSTRING(COALESCE({s}, ''), LENGTH(COALESCE({s}, '')), 1)), 0) * {self.primes[20]} ELSE 0 END)"
+        )
+
+        return (
+            f"RIGHT('00000000' + CONVERT(VARCHAR(8), (\n        {separator.join(parts)}\n      ) % 16777215), 16), 8)"
+        )
 
     def concat(self, items: List[str]) -> str:
         """Provide SQL for concatenating multiple columns into a string for Sybase IQ."""
@@ -227,13 +325,19 @@ class Dialect(BaseDialect):
         return " || ".join(items)
 
     def normalize_uuid(self, value: str, coltype: ColType_UUID) -> str:
+        s_temp = re.sub(r'["\[\]`]', "", value)
+        raw_col_info = self.get_column_raw_info(s_temp)
+        ch_len = (raw_col_info and raw_col_info.character_maximum_length) or None
+        if not ch_len:
+            ch_len = 2500
+        ch_len = max(ch_len, 2500)
         if isinstance(coltype, String_UUID):
             if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
-                return f"CAST({value} AS VARCHAR(2500))"  # IQ: Match column length
-            return f"CAST({value} AS NVARCHAR(5000))"  # ASE: Match column length
+                return f"CAST({value} AS VARCHAR({ch_len}))"  # IQ: Match column length
+            return f"CAST({value} AS NVARCHAR({ch_len}))"  # ASE: Match column length
         if self.sybase_driver_type.is_iq or self.query_config_for_free_tds["freetds_query_chosen"]:
-            return f"CONVERT(VARCHAR(36), {value})"
-        return f"CONVERT(NVARCHAR(36), {value})"
+            return f"CONVERT(VARCHAR({ch_len}), {value})"
+        return f"CONVERT(NVARCHAR({ch_len}), {value})"
 
     def parse_table_name(self, name: str) -> DbPath:
         "Parse the given table name into a DbPath"
@@ -273,7 +377,6 @@ class Sybase(ThreadedDatabase):
             self.dialect.default_schema = self.default_schema
         except KeyError:
             raise ValueError("Specify a default database and schema.")
-
         self._sybase = import_sybase()
         self._detect_driver_type(self._args.get("driver", None))
         self._conn = self.create_connection()
@@ -286,6 +389,7 @@ class Sybase(ThreadedDatabase):
         username = self._args.get("user", None)
         password = self._args.get("password", None)
         driver = self._args.get("driver", None)
+        max_query_timeout = 60 * 5  # 300 seconds
 
         if self.dialect.sybase_driver_type.is_freetds:
             conn_dict = {
@@ -298,10 +402,10 @@ class Sybase(ThreadedDatabase):
             }
 
             conn_dict["host"] = host or server
-
             try:
                 logger.debug(f"Attempting FreeTDS connection with config: {conn_dict}")
                 self._conn = self._sybase.connect(**conn_dict)
+                self._conn.timeout = max_query_timeout
                 logger.info(f"Successfully connected to Sybase using FreeTDS: {conn_dict}")
                 return self._conn
             except Exception as e:
@@ -315,7 +419,6 @@ class Sybase(ThreadedDatabase):
             "UID": username,
             "PWD": password,
         }
-
         connection_attempts = []
         if self.dialect.sybase_driver_type.is_ase:
             connection_attempts = [
@@ -379,6 +482,7 @@ class Sybase(ThreadedDatabase):
                     try:
                         logger.debug(f"Attempting connection with config: {final_config}")
                         self._conn = self._sybase.connect(**final_config)
+                        self._conn.timeout = max_query_timeout
                         logger.info(
                             f"Successfully connected to Sybase using: "
                             f"driver={driver}, "
@@ -479,7 +583,13 @@ class Sybase(ThreadedDatabase):
                     self.dialect.query_config_for_free_tds["ase_query_chosen"] = True
                     return ase_query
                 else:
+                    max_temp_space_usage_query = "SET TEMPORARY OPTION MAX_TEMP_SPACE_PER_CONNECTION = 5120"
+                    if self._query_cursor(self._conn.cursor(), max_temp_space_usage_query, test_query=True):
+                        logger.info("Max temporary space usage set successfully.")
+                    else:
+                        logger.warning("Failed to set max temporary space usage, continuing with default settings.")
                     logger.info("Sybase IQ Detected")
+
                     self.dialect.query_config_for_free_tds["freetds_query_chosen"] = True
                     return iq_query
             except Exception as e:
