@@ -522,7 +522,7 @@ class HashDiffer(TableDiffer):
 
         if count1 == 0 and count2 == 0:
             logger.debug(
-                "Uneven distribution of keys detected in segment %s..%s (big gaps in the key column). "
+                "Uneven distribution of keys detected in segment {}..{} (big gaps in the key column). "
                 "For better performance, we recommend to increase the bisection-threshold.",
                 table1.min_key,
                 table1.max_key,
