@@ -14,6 +14,7 @@
 
 import logging
 import time
+from decimal import Decimal
 from itertools import product
 from typing import Container, Dict, List, Optional, Sequence, Tuple
 
@@ -24,14 +25,35 @@ from typing_extensions import Self
 
 from data_diff.abcs.database_types import DbKey, DbPath, DbTime, IKey, NumericType
 from data_diff.databases.base import Database
-from data_diff.queries.api import SKIP, Code, Count, Expr, max_, min_, table, this
+from data_diff.queries.api import (
+    SKIP,
+    Code,
+    Count,
+    Expr,
+    and_,
+    max_,
+    min_,
+    or_,
+    table,
+    this,
+)
 from data_diff.queries.extras import (
     ApplyFuncAndNormalizeAsString,
     Checksum,
     NormalizeAsString,
 )
 from data_diff.schema import RawColumnInfo, Schema, create_schema
-from data_diff.utils import ArithString, Vector, safezip, split_space
+from data_diff.utils import (
+    ArithDate,
+    ArithDateTime,
+    ArithString,
+    ArithTimestamp,
+    ArithTimestampTZ,
+    ArithUnicodeString,
+    Vector,
+    safezip,
+    split_space,
+)
 
 # logger = logging.getLogger("table_segment")
 
@@ -44,10 +66,18 @@ def split_key_space(min_key: DbKey, max_key: DbKey, count: int) -> List[DbKey]:
     if max_key - min_key <= count:
         count = 1
 
-    if isinstance(min_key, ArithString):
+    # Handle arithmetic string types (including temporal types)
+    if isinstance(
+        min_key, (ArithString, ArithUnicodeString, ArithDateTime, ArithDate, ArithTimestamp, ArithTimestampTZ)
+    ):
         assert type(min_key) is type(max_key)
         checkpoints = min_key.range(max_key, count)
     else:
+        # Handle numeric types
+        if isinstance(min_key, Decimal):
+            min_key = float(min_key)
+        if isinstance(max_key, Decimal):
+            max_key = float(max_key)
         checkpoints = split_space(min_key, max_key, count)
 
     assert all(min_key < x < max_key for x in checkpoints)
@@ -282,19 +312,65 @@ class TableSegment:
 
         return result
 
-    def get_sample_data(self, limit: int = 100) -> list:
-        "Download all the relevant values of the segment from the database"
+    # def get_sample_data(self, limit: int = 100) -> list:
+    #     "Download all the relevant values of the segment from the database"
 
-        exprs = []
-        for c in self.key_columns:
-            quoted = self.database.dialect.quote(c)
-            exprs.append(NormalizeAsString(Code(quoted), self._schema[c]))
-        if self.where:
-            select = (
-                self.source_table.select(*self._relevant_columns_repr).where(self.where).order_by(*exprs).limit(limit)
-            )
+    #     exprs = []
+    #     for c in self.key_columns:
+    #         quoted = self.database.dialect.quote(c)
+    #         exprs.append(NormalizeAsString(Code(quoted), self._schema[c]))
+    #     if self.where:
+    #         select = self.source_table.select(*self._relevant_columns_repr).where(Code(self._where())).limit(limit)
+    #         self.key_columns
+    #     else:
+    #         select = self.source_table.select(*self._relevant_columns_repr).limit(limit)
+
+    #     start_time = time.monotonic()
+    #     result = self.database.query(select, List[Tuple])
+    #     query_time_ms = (time.monotonic() - start_time) * 1000
+    #     self._update_stats("row_fetch_queries_stats", query_time_ms)
+
+    def get_sample_data(self, limit: int = 100, sample_keys: Optional[List[List[DbKey]]] = None) -> list:
+        """
+        Download relevant values of the segment from the database.
+        If `sample_keys` is provided, it filters rows matching those composite keys.
+
+        Parameters:
+            limit (int): Maximum number of rows to return (default: 100).
+            sample_keys (Optional[List[List[DbKey]]]): List of composite keys to filter rows.
+                Each inner list must match the number of key_columns.
+
+        Returns:
+            list: List of tuples containing the queried row data.
+        """
+        select = self.make_select().select(*self._relevant_columns_repr)
+
+        filters = []
+
+        if sample_keys:
+            key_exprs = []
+            for key_values in sample_keys:
+                and_exprs = []
+                for col, val in safezip(self.key_columns, key_values):
+                    quoted = self.database.dialect.quote(col)
+                    schema = self._schema[col]
+                    if val is None:
+                        and_exprs.append(Code(quoted + " IS NULL"))
+                        continue
+                    mk_v = schema.make_value(val)
+                    constant_val = self.database.dialect._constant_value(mk_v)
+                    where_expr = f"{quoted} = {constant_val}"
+                    and_exprs.append(Code(where_expr))
+                if and_exprs:
+                    key_exprs.append(and_(*and_exprs))
+            if key_exprs:
+                filters.append(or_(*key_exprs))
+        if filters or self.where:
+            select = select.where(*filters)
         else:
-            select = self.source_table.select(*self._relevant_columns_repr).order_by(*exprs).limit(limit)
+            logger.warning("No filters applied; fetching up to {} rows without key restrictions", limit)
+
+        select = select.limit(limit)
 
         start_time = time.monotonic()
         result = self.database.query(select, List[Tuple])
@@ -315,7 +391,6 @@ class TableSegment:
 
     def segment_by_checkpoints(self, checkpoints: List[List[DbKey]]) -> List["TableSegment"]:
         "Split the current TableSegment to a bunch of smaller ones, separated by the given checkpoints"
-
         return [self.new_key_bounds(min_key=s, max_key=e) for s, e in create_mesh_from_points(*checkpoints)]
 
     def new(self, **kwargs) -> Self:
